/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_TRI_MESH_H
#define _GCTL_TRI_MESH_H

#include "mesh.h"

namespace gctl
{
    class triangle_mesh : public base_mesh
    {
    public:

        /**
         * 以下为mesh类型的虚函数实现
         */

        void load_gmsh(std::string filename, index_packed_e packed = Packed);
        void save_gmsh(std::string filename, index_packed_e packed = Packed);

        void load_binary(std::string filename);
        void save_binary(std::string filename);

        void show_mesh_dimension(std::ostream &os) const;

        /**
         * 以下为triangle_mesh的专有函数
         */

        triangle_mesh();
        triangle_mesh(std::string in_name, std::string in_info, const array<vertex3dc> &in_nodes, 
            const array<triangle> &in_triangles);
        virtual ~triangle_mesh();

        void init(std::string in_name, std::string in_info, const array<vertex3dc> &in_nodes, 
            const array<triangle> &in_triangles);

        const array<vertex3dc> &get_nodes() const;
        const array<triangle> &get_elements() const;

        void load_gmsh_groups();

        /**
         * @brief 将模型物理分组转换为模型数据组	
         * 
         * @param datname 赋值给模型数据组的名称（如果数组不存在则新建）
         * @param phynames 物理组名称
         * @param phyvals 物理组值
         */
        void groups2data(std::string datname, _1s_vector phynames, _1d_vector phyvals);

    protected:
        array<vertex3dc> nodes_;
        array<triangle> elems_;
        array<gmsh_physical_group> groups_;
        _2i_vector elems_tag_;
    };
};

#endif //_GCTL_TRI2D_MESH_H